@@ -33,12 +33,28 @@
 
 #include <QtCore/qdebug.h>
 #include <QtCore/qlist.h>
-#include <QtCore/qstringlistmodel.h>
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
+
 namespace qdesigner_internal {
 
+// Validator for theme line edit, accepts empty or non-blank strings.
+class BlankSuppressingValidator : public QValidator {
+public:
+    explicit BlankSuppressingValidator(QObject * parent = nullptr) : QValidator(parent) {}
+    State validate(QString &input, int &pos) const override
+    {
+        const auto blankPos = input.indexOf(u' ');
+        if (blankPos != -1) {
+            pos = blankPos;
+            return Invalid;
+        }
+        return Acceptable;
+    }
+};
+
 // -------------------- LanguageResourceDialogPrivate
 class LanguageResourceDialogPrivate {
     LanguageResourceDialog *q_ptr;
@@ -76,13 +92,14 @@ void LanguageResourceDialogPrivate::init(LanguageResourceDialog *p)
     QLayout *layout = new QVBoxLayout(p);
     layout->addWidget(m_browser);
     layout->addWidget(m_dialogButtonBox);
-    QObject::connect(m_dialogButtonBox, SIGNAL(accepted()), p, SLOT(slotAccepted()));
+    QObject::connect(m_dialogButtonBox, &QDialogButtonBox::accepted, p, [this] { slotAccepted(); });
     QObject::connect(m_dialogButtonBox, &QDialogButtonBox::rejected, p, &QDialog::reject);
-    QObject::connect(m_browser, SIGNAL(currentPathChanged(QString)), p, SLOT(slotPathChanged(QString)));
-    QObject::connect(m_browser, SIGNAL(pathActivated(QString)), p, SLOT(slotAccepted()));
+    QObject::connect(m_browser, &QDesignerResourceBrowserInterface::currentPathChanged,
+                     p, [this](const QString &fileName) { slotPathChanged(fileName); });
+    QObject::connect(m_browser, &QDesignerResourceBrowserInterface::pathActivated,
+                     p, [this] { slotAccepted(); });
     p->setModal(true);
     p->setWindowTitle(LanguageResourceDialog::tr("Choose Resource"));
-    p->setWindowFlags(p->windowFlags() & ~Qt::WindowContextHelpButtonHint);
     setOkButtonEnabled(false);
 }
 
@@ -145,13 +162,6 @@ LanguageResourceDialog* LanguageResourceDialog::create(QDesignerFormEditorInterf
 
 // ------------ IconSelectorPrivate
 
-static inline QPixmap emptyPixmap()
-{
-    QImage img(16, 16, QImage::Format_ARGB32_Premultiplied);
-    img.fill(0);
-    return QPixmap::fromImage(img);
-}
-
 class IconSelectorPrivate
 {
     IconSelector *q_ptr = nullptr;
@@ -308,19 +318,18 @@ static QString imageFilter()
 {
     QString filter = QApplication::translate("IconSelector", "All Pixmaps (");
     const auto supportedImageFormats = QImageReader::supportedImageFormats();
-    const QString jpeg = QStringLiteral("JPEG");
     const qsizetype count = supportedImageFormats.size();
     for (qsizetype i = 0; i < count; ++i) {
         if (i)
-            filter += QLatin1Char(' ');
-        filter += QStringLiteral("*.");
+            filter += u' ';
+        filter += "*."_L1;
         const QString outputFormat = QString::fromUtf8(supportedImageFormats.at(i));
-        if (outputFormat != jpeg)
+        if (outputFormat != "JPEG"_L1)
             filter += outputFormat.toLower();
         else
-            filter += QStringLiteral("jpg *.jpeg");
+            filter += "jpg *.jpeg"_L1;
     }
-    filter += QLatin1Char(')');
+    filter += u')';
     return filter;
 }
 
@@ -414,7 +423,7 @@ IconSelector::IconSelector(QWidget *parent) :
     d_ptr->m_resetAllAction = new QAction(tr("Reset All"), this);
     d_ptr->m_resetAction->setEnabled(false);
     d_ptr->m_resetAllAction->setEnabled(false);
-    //d_ptr->m_resetAction->setIcon(createIconSet(QString::fromUtf8("resetproperty.png")));
+    //d_ptr->m_resetAction->setIcon(createIconSet("resetproperty.png"_L1));
 
     setMenu->addAction(setResourceAction);
     setMenu->addAction(setFileAction);
@@ -424,7 +433,7 @@ IconSelector::IconSelector(QWidget *parent) :
 
     int index = 0;
     QStringList items;
-    for (const auto &item : qAsConst(d_ptr->m_stateToName)) {
+    for (const auto &item : std::as_const(d_ptr->m_stateToName)) {
         const QPair<QIcon::Mode, QIcon::State> state = item.first;
         const QString name = item.second;
 
@@ -437,13 +446,18 @@ IconSelector::IconSelector(QWidget *parent) :
 
     d_ptr->m_iconButton->setMenu(setMenu);
 
-    connect(d_ptr->m_stateComboBox, SIGNAL(activated(int)), this, SLOT(slotStateActivated()));
-    connect(d_ptr->m_iconButton, SIGNAL(clicked()), this, SLOT(slotSetActivated()));
-    connect(setResourceAction, SIGNAL(triggered()), this, SLOT(slotSetResourceActivated()));
-    connect(setFileAction, SIGNAL(triggered()), this, SLOT(slotSetFileActivated()));
-    connect(d_ptr->m_resetAction, SIGNAL(triggered()), this, SLOT(slotResetActivated()));
-    connect(d_ptr->m_resetAllAction, SIGNAL(triggered()), this, SLOT(slotResetAllActivated()));
-
+    connect(d_ptr->m_stateComboBox, &QComboBox::activated,
+            this, [this] { d_ptr->slotStateActivated(); });
+    connect(d_ptr->m_iconButton, &QAbstractButton::clicked,
+            this, [this] { d_ptr->slotSetActivated(); });
+    connect(setResourceAction, &QAction::triggered,
+            this, [this] { d_ptr->slotSetResourceActivated(); });
+    connect(setFileAction, &QAction::triggered,
+            this, [this] { d_ptr->slotSetFileActivated(); });
+    connect(d_ptr->m_resetAction, &QAction::triggered,
+            this, [this] { d_ptr->slotResetActivated(); });
+    connect(d_ptr->m_resetAllAction, &QAction::triggered,
+            this, [this] { d_ptr->slotResetAllActivated(); });
     d_ptr->slotUpdate();
 }
 
@@ -473,47 +487,47 @@ void IconSelector::setFormEditor(QDesignerFormEditorInterface *core)
 void IconSelector::setIconCache(DesignerIconCache *iconCache)
 {
     d_ptr->m_iconCache = iconCache;
-    connect(iconCache, SIGNAL(reloaded()), this, SLOT(slotUpdate()));
+    connect(iconCache, &DesignerIconCache::reloaded, this, [this] { d_ptr->slotUpdate(); });
     d_ptr->slotUpdate();
 }
 
 void IconSelector::setPixmapCache(DesignerPixmapCache *pixmapCache)
 {
     d_ptr->m_pixmapCache = pixmapCache;
-    connect(pixmapCache, SIGNAL(reloaded()), this, SLOT(slotUpdate()));
+    connect(pixmapCache, &DesignerPixmapCache::reloaded, this, [this] { d_ptr->slotUpdate(); });
     d_ptr->slotUpdate();
 }
 
 // --- IconThemeEditor
 
+static const QMap<QString, QIcon> &themeIcons()
+{
+   static QMap<QString, QIcon> result;
+   if (result.isEmpty()) {
+       QFile file(u":/qt-project.org/designer/icon-naming-spec.txt"_s);
+       if (file.open(QIODevice::ReadOnly)) {
+           while (!file.atEnd()) {
+               const auto line = file.readLine().trimmed();
+               if (line.isEmpty() || line.startsWith('#'))
+                   continue;
+               const auto iconName = QString::fromUtf8(line);
+               result.insert(iconName, QIcon::fromTheme(iconName));
+           }
+           file.close();
+       }
+   }
+   return result;
+}
+
 struct IconThemeEditorPrivate {
     IconThemeEditorPrivate();
 
-    const QPixmap m_emptyPixmap;
     QComboBox *m_themeComboBox;
-    QLabel *m_themeLabel;
-    QStringListModel *m_themeModel;
 };
 
 IconThemeEditorPrivate::IconThemeEditorPrivate() :
-    m_emptyPixmap(emptyPixmap()),
-    m_themeComboBox(new QComboBox),
-    m_themeLabel(new QLabel),
-    m_themeModel(new QStringListModel)
+    m_themeComboBox(new QComboBox)
 {
-    QStringList iconNames;
-    QFile file(QStringLiteral(":/qt-project.org/designer/icon-naming-spec.txt"));
-    if (file.open(QIODevice::ReadOnly)) {
-        while (!file.atEnd()) {
-            const auto line = file.readLine().trimmed();
-            if (line.isEmpty() || line.startsWith('#'))
-                continue;
-            iconNames.append(QString::fromUtf8(line));
-        }
-        file.close();
-    }
-    std::sort(iconNames.begin(), iconNames.end());
-    m_themeModel->setStringList(iconNames);
 }
 
 IconThemeEditor::IconThemeEditor(QWidget *parent, bool wantResetButton) :
@@ -522,26 +536,18 @@ IconThemeEditor::IconThemeEditor(QWidget *parent, bool wantResetButton) :
     QHBoxLayout *mainHLayout = new QHBoxLayout;
     mainHLayout->setContentsMargins(QMargins());
 
-    // Vertically center theme preview label
-    d->m_themeLabel->setPixmap(d->m_emptyPixmap);
-
-    QVBoxLayout *themeLabelVLayout = new QVBoxLayout;
-    d->m_themeLabel->setMargin(1);
-    themeLabelVLayout->setContentsMargins(QMargins());
-    themeLabelVLayout->addSpacerItem(new QSpacerItem(0, 0, QSizePolicy::Ignored, QSizePolicy::MinimumExpanding));
-    themeLabelVLayout->addWidget(d->m_themeLabel);
-    themeLabelVLayout->addSpacerItem(new QSpacerItem(0, 0, QSizePolicy::Ignored, QSizePolicy::MinimumExpanding));
-    mainHLayout->addLayout(themeLabelVLayout);
-
-    d->m_themeComboBox->setModel(d->m_themeModel);
+    const auto icons = themeIcons();
+    for (auto i = icons.constBegin(); i != icons.constEnd(); ++i)
+        d->m_themeComboBox->addItem(i.value(), i.key());
     d->m_themeComboBox->setCurrentIndex(-1);
-    connect(d->m_themeComboBox, &QComboBox::currentTextChanged, this, &IconThemeEditor::slotChanged);
+    d->m_themeComboBox->setEditable(true);
+    d->m_themeComboBox->lineEdit()->setValidator(new BlankSuppressingValidator(this));
     connect(d->m_themeComboBox, &QComboBox::currentTextChanged, this, &IconThemeEditor::edited);
     mainHLayout->addWidget(d->m_themeComboBox);
 
     if (wantResetButton) {
         QToolButton *themeResetButton = new QToolButton;
-        themeResetButton->setIcon(createIconSet(QStringLiteral("resetproperty.png")));
+        themeResetButton->setIcon(createIconSet("resetproperty.png"_L1));
         connect(themeResetButton, &QAbstractButton::clicked, this, &IconThemeEditor::reset);
         mainHLayout->addWidget(themeResetButton);
     }
@@ -558,23 +564,6 @@ void IconThemeEditor::reset()
     emit edited(QString());
 }
 
-void IconThemeEditor::slotChanged(const QString &theme)
-{
-    updatePreview(theme);
-}
-
-void IconThemeEditor::updatePreview(const QString &t)
-{
-    // Update preview label with icon.
-    if (t.isEmpty() || !QIcon::hasThemeIcon(t)) { // Empty
-        if (d->m_themeLabel->pixmap(Qt::ReturnByValue).cacheKey() != d->m_emptyPixmap.cacheKey())
-            d->m_themeLabel->setPixmap(d->m_emptyPixmap);
-    } else {
-        const QIcon icon = QIcon::fromTheme(t);
-        d->m_themeLabel->setPixmap(icon.pixmap(d->m_emptyPixmap.size()));
-    }
-}
-
 QString IconThemeEditor::theme() const
 {
     return d->m_themeComboBox->currentText();
